unit input;

interface

uses
  SysUtils, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QComCtrls, QExtCtrls, DateUtils;

type
  TDataInput = class(TForm)
    ListView1: TListView;
    InText: TEdit;
    RadioGroup1: TRadioGroup;
    DoCheck: TCheckBox;
    Label1: TLabel;
    Label2: TLabel;
    Button1: TButton;
    Button2: TButton;
    DateText1: TEdit;
    DateText2: TEdit;
    DateText3: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    CalDown: TButton;
    CalUp: TButton;
    CalDisp: TEdit;
    Calender: TImage;
    procedure DoCheckClick(Sender: TObject);
    procedure CalDownClick(Sender: TObject);
    procedure CalUpClick(Sender: TObject);
    procedure CalenderSelect(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private  }
    EndDateKeep : TDateTime;
    CalYear : Integer;
    CalMonth : Integer;
    CalArray : Integer;
    function GetMemo : String;
    procedure SetMemo(text : String);
    function GetEndFlag : Boolean;
    procedure SetEndFlag(flag : Boolean);
    function GetPriority : Integer;
    procedure SetPriority(num : Integer);
    function GetTargetDate : TDateTime;
    procedure SetTargetDate(date : TDateTime);
    function GetEndDate : TDateTime;
    procedure SetEndDate(date : TDateTime);
    procedure DispCalender;
  public
  published
    property Memo : String read GetMemo Write SetMemo;
    property EndFlag : Boolean read GetEndFlag Write SetEndFlag;
    property Priority : Integer read GetPriority Write SetPriority;
    property TargetDate : TDateTime read GetTargetDate Write SetTargetDate;
    property EndDate : TDateTime read GetEndDate Write SetEndDate;
  end;

var
  DataInput: TDataInput;

implementation

{$R *.xfm}

{--- ǡϤ }
function TDataInput.GetMemo : String;
begin
  Result := InText.Text;
end;

procedure TDataInput.SetMemo(text : String);
begin
  InText.Text := text;
end;

function TDataInput.GetEndFlag : Boolean;
begin
  Result := DoCheck.Checked;
end;

procedure TDataInput.SetEndFlag(flag : Boolean);
begin
  DoCheck.Checked := flag;
end;

function TDataInput.GetPriority : Integer;
begin
  Result := RadioGroup1.ItemIndex;
end;

procedure TDataInput.SetPriority(num : Integer);
begin
  RadioGroup1.ItemIndex := num;
end;

function TDataInput.GetTargetDate : TDateTime;
var
  datestring : string;
  t1, t2, t3 : Integer;
begin
//  result := DateTimePicker1.Date;
  {=== Kylix Ѥ˽ľ }
  try
    t1 := StrToInt(DateText1.Text);
  except
    t1 := 0;
  end;
  if t1 < 1970 then t1 := 1970;
  if t1 > 2048 then t1 := 2048;
  try
    t2 := StrToInt(DateText2.Text);
  except
    t2 := 0;
  end;
  if t2 < 1 then t2 := 1;
  if t2 > 12 then t2 := 12;
  try
    t3 := StrToInt(DateText3.Text);
  except
    t3 := 0;
  end;
  if t3 < 1 then t3 := 1;
  if t3 > DaysInAMonth(t1, t2) then t3 := DaysInAMonth(t1, t2);
  datestring := IntToStr(t1)+'/'+IntToStr(t2)+'/'+IntToStr(t3);
  result := StrToDate(datestring);
end;

procedure TDataInput.SetTargetDate(date : TDateTime);
var
  sdate : String;
  tmp : String;
begin
//  DateTimePicker1.Date := date;
  {=== Kylix Ѥ˽ľ }
  sdate := DateToStr(date);
  tmp := sdate[1]+sdate[2]+sdate[3]+sdate[4];
  DateText1.Text := tmp;
  CalYear := StrToInt(tmp);
  tmp := sdate[6]+sdate[7];
  DateText2.Text := tmp;
  CalMonth := StrToInt(tmp);
  tmp := sdate[9]+sdate[10];
  DateText3.Text := tmp;
  DispCalender;
end;

function TDataInput.GetEndDate : TDateTime;
begin
  result := EndDateKeep;
end;

procedure TDataInput.SetEndDate(date : TDateTime);
begin
  EndDateKeep := date;
end;


procedure TDataInput.DoCheckClick(Sender: TObject);
begin
  if DoCheck.Checked = True then EndDateKeep := Date;
end;



{==============================================}
{  ɽȥ            }
{==============================================}

procedure TDataInput.DispCalender;
var
  disptext: string;
  dt : TDateTime;
  dmax, col, cx, cy : integer;
  i : integer;
begin
  disptext := IntToStr(CalYear)+'/'+IntToStr(CalMonth);
  CalDisp.Text := disptext;
  disptext := disptext + '/01';
  dt := StrToDate(dispText);
  CalArray := DayOfTheWeek(dt);
  if CalArray = 7 then CalArray := 0;
  {=====  }
  Calender.Canvas.Pen.Mode := pmCopy;
  Calender.Canvas.Pen.Color := ClWhite;
  Calender.Canvas.Pen.Style := psSolid;
  Calender.Canvas.Pen.Width := 1;
  Calender.Canvas.Rectangle(0, 0, Calender.Width, Calender.Height);
  {--- դ񤭹 }
  col := CalArray;
  dmax := DaysInAMonth(CalYear, CalMonth);
  for i := 1 to dmax do
  begin
    if (col mod 7) = 0 then
      Calender.Canvas.Font.Color := ClRed
    else
      Calender.Canvas.Font.Color := ClBlack;
    cx := (col mod 7) * 19 + 2 + 2;
    cy := trunc(col / 7) * 21 + 1 + 3;
    if i < 10 then cx := cx + 6;
    Calender.Canvas.TextOut(cx, cy, IntToStr(i));
    col := col + 1;
  end;
  {---  }
  Calender.Canvas.Pen.Color := clGray;
  for i := 0 to 7 do
  begin
    Calender.Canvas.MoveTo((i*19)+2, 1);
    Calender.Canvas.LineTo((i*19)+2, 127);
  end;
  for i := 0 to 6 do
  begin
    Calender.Canvas.MoveTo(2, (i*21)+1);
    Calender.Canvas.LineTo(135, (i*21)+1);
  end;

  Calender.Refresh;

end;

procedure TDataInput.CalDownClick(Sender: TObject);
begin
  CalMonth := CalMonth - 1;
  if CalMonth = 0 then
  begin
    CalMonth := 12;
    CalYear := CalYear - 1;
    if CalYear = 1969 then Calyear := 1970;
  end;
  DispCalender;
end;

procedure TDataInput.CalUpClick(Sender: TObject);
begin
  CalMonth := CalMonth + 1;
  if CalMonth = 13 then
  begin
    CalMonth := 1;
    CalYear := CalYear + 1;
    if CalYear = 2049 then Calyear := 2048;
  end;
  DispCalender;
end;

{ ----- å줿Τ }
procedure TDataInput.CalenderSelect(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  pos : Integer;
begin
  if (x < 2) or (x > 135) then Exit;
  if (y < 1) or (y > 127) then Exit;
  pos := trunc((x - 2) / 19) + (trunc((y - 1) / 21) * 7) + 1;
  pos := pos - CalArray;
  if pos < 1 then Exit;
  if pos > DaysInAMonth(CalYear, CalMonth) then Exit;
  { -- ̤ȿ }
  DateText1.Text := IntToStr(CalYear);
  DateText2.Text := IntToStr(CalMonth);
  DateText3.Text := IntToStr(pos);
end;

end.
